/*******************************************************************************
*
*	TITLE:		disabled.c 
*
*	VERSION:	0.1 (Beta)                           
*
*	DATE:		31-Dec-2007
*
*	AUTHOR:		R. Kevin Watson
*				kevinw@jpl.nasa.gov
*
*	COMMENTS:	This file best viewed with tabs set to four.
*
*				You are free to use this source code for any non-commercial
*				use. Please do not make copies of this source code, modified
*				or un-modified, publicly available on the internet or elsewhere
*				without permission. Thanks.
*
*				Copyright 2007-2008 R. Kevin Watson. All rights are reserved.
*
********************************************************************************
*
*	Change log:
*
*	DATE         REV  DESCRIPTION
*	-----------  ---  ----------------------------------------------------------
*	31-Dec-2007  0.1  RKW Original
*
*******************************************************************************/

#include "ifi_frc.h"
#include "pwm.h"
#include "gyro.h"
#include "timers.h"
#include "interrupts.h"
#include "serial_ports.h"
#include "ifi_code.h"
#include "disabled.h"
#include "teleop.h"
#include "adc.h"

/*******************************************************************************
*
*	FUNCTION:		Disabled_Init()
*
*	PURPOSE:		This is where you put code that needs to execute
*					just once at the start of disabled mode.			
* 
*	CALLED FROM:	main() in ifi_frc.c
*
*	PARAMETERS:		None
*
*	RETURNS:		Nothing
* 
*	COMMENTS:		While in this mode, all operator interface data is valid
*					and all robot controller outputs (PWMs and relays) are
*					disabled.
*
*******************************************************************************/
extern unsigned int arm_offset;
void Disabled_Init(void)
{
    static char init_flag=1;

    //IR_OFF;
    IR_ON; //Doesn't matter, outputs are disabled of course
    SHIFT_HIGH;

    // Reset the pot starting position if the arm is at Stow upon startup
//    typical arm_offset value is 1356
//    if (ARM_LOWER_LIMIT == 0)
//    {
//        printf("ARM_LOWER_LIMIT=%d, ",ARM_LOWER_LIMIT);
//        arm_offset = ARM_POT;
//        printf("arm_offset=%d\r\n", (int)arm_offset);
//    }
}

/*******************************************************************************
*
*	FUNCTION:		Disabled()
*
*	PURPOSE:		This is where you put code that you want to execute while
*					your robot is disabled.	While in autonomous mode, this 
*					function is called every 26.2ms when after data is received
*					from the master processor.			
* 
*	CALLED FROM:	main() in ifi_frc.c
*
*	PARAMETERS:		None
*
*	RETURNS:		Nothing
* 
*	COMMENTS:		While in this mode, all operator interface data is valid
*					and all robot controller outputs (PWMs and relays) are
*					disabled.
*
*******************************************************************************/
void Disabled(void)
{
	static unsigned int j = 0;
	int temp_gyro_bias;
    static int counter=0;

    if(j < 1000)	j++;

	if(j == 10)
	{
	    printf("\r\nCalculating Gyro Bias...\r\n");
	}
	if(j == 38) // let the gyro stablize for a second before starting a calibration
	{
		// start a gyro bias calculation
		Start_Gyro_Bias_Calc();
	}
	if(j == 191) // allow calibration routine to run for four seconds 
	{
		// terminate the gyro bias calculation
		Stop_Gyro_Bias_Calc();

		// reset the gyro heading angle
		Reset_Gyro_Angle();

		temp_gyro_bias = Get_Gyro_Bias();
        if (temp_gyro_bias < 1500) 
            GyroStatus=NOGYRO;
        else
            GyroStatus=GYRO;
		printf("Gyro Bias=%d\r\n", temp_gyro_bias);
	}

    counter++;
    if (counter == 38)
    {
        //printf("dig11 = %d, dig12=%d, dig13=%d\r\n", rc_dig_in11, rc_dig_in12,rc_dig_in13);
        //printf("Arm sensor=%d, ARM POT=%d\n\r",(int)rc_dig_in07, (int)ARM_POT);
		counter = 0;
    }

}

/*******************************************************************************
*
*	FUNCTION:		Disabled_Spin()
*
*	PURPOSE:		While in disabled mode, this function is called
*					continuously between calls to Disabled().			
* 
*	CALLED FROM:	main() in ifi_frc.c
*
*	PARAMETERS:		None
*
*	RETURNS:		Nothing
* 
*	COMMENTS:		While in this mode, all operator interface data is valid
*					and all robot controller outputs (PWMs and relays) are
*					disabled.
*
*******************************************************************************/
void Disabled_Spin(void)
{
	 Process_Gyro_Data();
}
