/*******************************************************************************
* FILE NAME: ifi_code.c
*
* DESCRIPTION:
*  This file contains some useful functions that you can call in your program.
*
*******************************************************************************/

#include "ifi_frc.h"
#include "ifi_code.h"
#include "teleop.h"



/*******************************************************************************
*
*	FUNCTION:		Update_OI_LEDs()
*
*	PURPOSE:		Updates the state of the various user
*					defined	LEDs on the operator interface.	
* 
*	CALLED FROM:
*
*	PARAMETERS:		None
*
*	RETURNS:		Nothing
* 
*	COMMENTS:
*
*******************************************************************************/
#define ON 1
#define OFF 0
void Update_OI_LEDs(void)
{
	if(user_display_mode == 0)
	{
        if (LEFT_DRIVE > (127-10) )
         Pwm1_green  = ON;
        else
          Pwm1_green  = OFF;
        if (LEFT_DRIVE <  (127+10) )
          Pwm1_red  = ON;
        else
          Pwm1_red  = OFF;

        if (RIGHT_DRIVE > (127-10) )
         Pwm2_green  = ON;
        else
          Pwm2_green  = OFF;
        if (RIGHT_DRIVE <  (127+10) )
          Pwm2_red  = ON;
        else
          Pwm2_red  = OFF;

        if (ARM_DRIVE > (127-10) )
            Relay1_green  = ON;
        else
            Relay1_green  = OFF;
        if (ARM_DRIVE <  (127+10) )
            Relay1_red  = ON;
        else
            Relay1_red  = OFF;

		Relay2_green  = OFF;
		Relay2_red  = OFF;

		// update the "Switch 1", "Switch 2" and "Switch 3" LEDs
		Switch1_LED = 0; //Unused
		//Switch2_LED = 0; //IR Receiver on or off
		Switch3_LED = relay8_fwd; // Compressor on or off;
	}
  	else  /* User Mode is On - displays data in OI 4-digit display*/
	{
		//User_Mode_byte = backup_voltage * 10;
	}
}   


/*******************************************************************************
* FUNCTION NAME: Limit_Switch_Max
* PURPOSE:       Sets a PWM value to neutral (127) if it exceeds 127 and the
*                limit switch is on.
* CALLED FROM:   this file
* ARGUMENTS:     
*     Argument       Type             IO   Description
*     --------       -------------    --   -----------
*     switch_state   unsigned char    I    limit switch state
*     *input_value   pointer           O   points to PWM byte value to be limited
* RETURNS:       void
*******************************************************************************/
void Limit_Switch_Max(unsigned char switch_state, unsigned char *input_value)
{
  if (switch_state == CLOSED)
  { 
    if(*input_value > 127)
      *input_value = 127;
  }
}


/*******************************************************************************
* FUNCTION NAME: Limit_Switch_Min
* PURPOSE:       Sets a PWM value to neutral (127) if it's less than 127 and the
*                limit switch is on.
* CALLED FROM:   this file
* ARGUMENTS:     
*     Argument       Type             IO   Description
*     --------       -------------    --   -----------
*     switch_state   unsigned char    I    limit switch state
*     *input_value   pointer           O   points to PWM byte value to be limited
* RETURNS:       void
*******************************************************************************/
void Limit_Switch_Min(unsigned char switch_state, unsigned char *input_value)
{
  if (switch_state == CLOSED)
  { 
    if(*input_value < 127)
      *input_value = 127;
  }
}


/*******************************************************************************
* FUNCTION NAME: Limit_Mix
* PURPOSE:       Limits the mixed value for one joystick drive.
* CALLED FROM:   Default_Routine, this file
* ARGUMENTS:     
*     Argument             Type    IO   Description
*     --------             ----    --   -----------
*     intermediate_value    int    I    
* RETURNS:       unsigned char
*******************************************************************************/
unsigned char Limit_Mix (int intermediate_value)
{
  static int limited_value;
  
  if (intermediate_value < 2000)
  {
    limited_value = 2000;
  }
  else if (intermediate_value > 2254)
  {
    limited_value = 2254;
  }
  else
  {
    limited_value = intermediate_value;
  }
  return (unsigned char) (limited_value - 2000);
}
