/*******************************************************************************
*
*	TITLE:		teleop.c 
*
*	VERSION:	0.1 (Beta)                           
*
*	DATE:		31-Dec-2007
*
*	AUTHOR:		R. Kevin Watson
*				kevinw@jpl.nasa.gov
*
*	COMMENTS:	This file best viewed with tabs set to four.
*
*				You are free to use this source code for any non-commercial
*				use. Please do not make copies of this source code, modified
*				or un-modified, publicly available on the internet or elsewhere
*				without permission. Thanks.
*
*				Copyright 2007-2008 R. Kevin Watson. All rights are reserved.
*
********************************************************************************
*
*	Change log:
*
*	DATE         REV  DESCRIPTION
*	-----------  ---  ----------------------------------------------------------
*	31-Dec-2007  0.1  RKW Original
*
*******************************************************************************/

#include "ifi_frc.h"
#include "adc.h"
#include "gyro.h"
#include "pwm.h"
#include "timers.h"
#include "interrupts.h"
#include "encoder.h"
#include "serial_ports.h"
#include "ifi_code.h"
#include "teleop.h"

void DropKick(void);
void Arm_Position(void);

char DropToRun=0; //After Hurdle immediately drop arm to Carry position
char GyroStatus=NOGYRO; //Is the gyro working correctly or not?

//Arm Pot range is 0-4088
//Off by 75 with current KP
// 188, 353,828,923, 968
// ARM_PICKUP puts the center of the rings 23" off the floor
unsigned int arm_offset = 1077; // 1100=stowed reading 1192  (20 ~= 3 inches)



/*******************************************************************************
*
*	FUNCTION:		Initialization()
*
*	PURPOSE:		This function is called once when the robot controller
*					is cold or warm booted. You should initialize your code
*					here.		
* 
*	CALLED FROM:	main() in ifi_frc.c
*
*	PARAMETERS:		None
*
*	RETURNS:		Nothing
* 
*	COMMENTS:
*
*******************************************************************************/
void Initialization(void)
{
	// Setup the digital I/O pins. Use "INPUT" to setup the pin
	// as an input and "OUTPUT" to setup the pin as an output.
	digital_io_01 = INPUT;
	digital_io_02 = INPUT;
	digital_io_03 = INPUT;
	digital_io_04 = INPUT;
	digital_io_05 = INPUT;
	digital_io_06 = INPUT;
	digital_io_07 = INPUT;
	digital_io_08 = INPUT;
	digital_io_09 = INPUT;
	digital_io_10 = INPUT;
	digital_io_11 = INPUT;
	digital_io_12 = INPUT;
	digital_io_13 = INPUT;
	digital_io_14 = INPUT;
	digital_io_15 = INPUT;
	digital_io_16 = INPUT;
	digital_io_17 = INPUT;
	digital_io_18 = INPUT;


	// Initialize the digital outputs. If the pin is configured
	// as an input above, it doesn't matter what state you
	// initialize it to here.
	rc_dig_out01 = 0;
	rc_dig_out02 = 0;
	rc_dig_out03 = 0;
	rc_dig_out04 = 0;
	rc_dig_out05 = 0;
	rc_dig_out06 = 0;
	rc_dig_out07 = 0;
	rc_dig_out08 = 0;
	rc_dig_out09 = 0;
	rc_dig_out10 = 0;
	rc_dig_out11 = 0;
	rc_dig_out12 = 0;
	rc_dig_out13 = 0;
	rc_dig_out14 = 0;
	rc_dig_out15 = 0;
	rc_dig_out16 = 0;
	rc_dig_out17 = 0;
	rc_dig_out18 = 0;

	// Initialize timers and interrupts here


	// Remove the // below to initialize the analog to digital converter (ADC) code
	 Initialize_ADC();

	// Remove the // below to initialize the gyro code (ADC code must be enabled too)
	 Initialize_Gyro();

	// Remove the // below to initialize encoder #1
	 Initialize_Encoder_1();

	// Remove the // below to initialize encoder #2
	 Initialize_Encoder_2();

	// Remove the // below to initialize encoders #3 through #6
	// Initialize_Encoder_3_6();

	printf("IFI User Processor Initialized ...\r\n");
}

/*******************************************************************************
*
*	FUNCTION:		Teleop_Init()
*
*	PURPOSE:		This is where you put code that needs to execute
*					just once at the start of teleoperation mode.		
* 
*	CALLED FROM:	main() in ifi_frc.c
*
*	PARAMETERS:		None
*
*	RETURNS:		Nothing
* 
*	COMMENTS:		While in this mode, all operator interface data is valid
*					and all robot controller outputs are enabled.
*
*******************************************************************************/
void Teleop_Init(void)
{
    IR_OFF;
}

/*******************************************************************************
*
*	FUNCTION:		Teleop()
*
*	PURPOSE:		This is where you put code that you want to execute while
*					your robot is in teleoperation mode. While in teleoperation
*					mode, this function is called every	26.2ms after new data
*					is received from the master processor.		
* 
*	CALLED FROM:	main() in ifi_frc.c
*
*	PARAMETERS:		None
*
*	RETURNS:		Nothing
* 
*	COMMENTS:		While in this mode, all operator interface data is valid
*					and all robot controller outputs are enabled.
*
*******************************************************************************/
//#define LEFTBALANCE 10  //Balance the left/right drivetrains
#define RIGHTBALANCE 10  //Balance the left/right drivetrains

void Teleop(void)
{
  static char grabber1_toggle=1, auto1_toggle=1, flutter_toggle=1;
  static char done=START, edone=START, e2done=START, e3done=START;
  static int heading=450, distance = 238;
  long drive;
  static unsigned char stick=0, gyro_turn=FALSE, encoder_drive=FALSE;
	long temp_gyro_angle;
    unsigned int desired_joint=0;
    static int wait_counter=0, wait_counter2=0, wait_counter3=0, counter=0, timed_grab=0;

	static unsigned int i = 0;
	long Encoder_Count;

// AndyMark encoders
// Forward 180": E1=2386, E2=2374
// Reverse 180": E1=-2336, E2=-2324 (184"=Forward counts of 2386/2374)

//Arm Pot 1224=stow, 1527=pickup, 1812=carry, 2570=hurdle, 2320=knockoff, cap=2570
//Arm Pot range is 0-4088

//Pneumatics (w/4 accumulators)" 1# for shifters, 5#=pushdown pads, 10# kicker, 10#=grabber
//One minute to charge 4 accumulators
      
// *********** Base driver ***********
    // Joystick Deadband
    if (LEFT_DRIVE > (127-10) && LEFT_DRIVE < (127+10))
    {
  	  LEFT_DRIVE = 127;
    }
    if (RIGHT_DRIVE > (127-10) && RIGHT_DRIVE < (127+10))
    {
	  RIGHT_DRIVE = 127;
    }


    //Robot pulls to the left, so slow down the right motors to compensate
    //drive = (int)LEFT_DRIVE-127;
    //drive -= (LEFTBALANCE*drive/127) + 127;
    //LEFT_MOTOR_1  = LEFT_MOTOR_2  = drive;
    LEFT_MOTOR_1  = LEFT_MOTOR_2  = LEFT_DRIVE;
    //RIGHT_MOTOR_1 = RIGHT_MOTOR_2 = RIGHT_DRIVE;
    drive = (int)RIGHT_DRIVE-127;
    drive -= (RIGHTBALANCE*drive/127) + 127;
    RIGHT_MOTOR_1 = RIGHT_MOTOR_2 = drive;


    // Gear shift
    if (GEAR_SHIFT_LOW)
    {
	  SHIFT_LOW;
    }
    if (GEAR_SHIFT_HIGH)
    {
	  SHIFT_HIGH;
    }


// Grabber
  if(GRABBER && grabber1_toggle)
  {
	grabber1_toggle = 0;
	BALL_TOGGLE;
  }
  if(!GRABBER)
  {
	grabber1_toggle = 1;
  }

  // Drop kick the ball
  // Lockout kicker unless in "Hurdle" position or "Manual"
    DropKick();

  //Set the Arm position
    Arm_Position();

  // ****** Compressor ******
    relay8_rev = 0;
    relay8_fwd = !rc_dig_in14;
    
    // grabber uses 16 lbs of air, 8 secs. to recover
    // flutter uses 10 lbs
    // shifter uses 1 lb
    // kicker & arm open combo uses 30 lbs, 15 sec to recover

    //User_Mode_byte = p3_wheel>>2;
	i++; // Let me count the ways
	// reset the loop counter
	if(i >= 19)
	{
	    temp_gyro_angle = Get_Gyro_Angle();
	    //printf("Gyro Angle=%d\r\n", (int)temp_gyro_angle);
		//Encoder_Count = Get_Encoder_1_Count();
		//printf("Left=%d, ",(int)Encoder_Count);	
		//Encoder_Count = Get_Encoder_2_Count();
	    //printf("Right=%d\r\n",(int)Encoder_Count);
        //printf("IR1=%d, IR2=%d, IR3=%d, IR4=%d, ",IR1,IR2,IR3,IR4);
        //printf("LEFT_MOTOR_1=%d, ",LEFT_MOTOR_1);
        //printf("AUTOMODE=%d, ",AUTOMODE);
        //printf("FieldL=%d,FC=%d,FR=%d, ",FIELD_LEFT, FIELD_CENTER, FIELD_RIGHT);
        printf("Arm sensor=%d, ARM POT=%d\n\r",(int)rc_dig_in07, (int)ARM_POT);
        //printf("dig11 = %d, dig12=%d, dig13=%d\r\n", rc_dig_in11, rc_dig_in12,rc_dig_in13);
        //printf("PUNCHER=%d\r\n", PUNCHER);
		i = 0;
	}

	// enable this to use IFI's default robot code
	// Default_Routine();	// located in ifi_code.c

	// update the state of the LEDs on the operator interface
	Update_OI_LEDs();		// located in ifi_code.c
}

/*******************************************************************************
*
*	FUNCTION:		Teleop_Spin()
*
*	PURPOSE:		While in teleoperation mode, this function is called
*					continuously between calls to Teleop().		
* 
*	CALLED FROM:	main() in ifi_frc.c
*
*	PARAMETERS:		None
*
*	RETURNS:		Nothing
* 
*	COMMENTS:		While in this mode, all operator interface data is valid
*					and all robot controller outputs are enabled.
*
*******************************************************************************/
void Teleop_Spin(void)
{
	// enable this function if you want to process gyro data during teleop mode
	 Process_Gyro_Data();
}


/*******************************************************************************
*	FUNCTION:		DropKick()
*	PURPOSE:		Drop kick the Ball
*******************************************************************************/
void DropKick(void)
{
  static char kicker_toggle = 1;
  static unsigned char ball_counter=0, punch_counter=0;

    //User_Mode_byte = (p3_wheel>>2)-12;

    // Puncher
    if (!PUNCHER && (punch_counter==1))
    {
        RETRACT;
        punch_counter = 0;
        DropToRun = 1; //Force arm to Carry position immediately
    }
    if (punch_counter > 1)
    {
        punch_counter--;
    }
    if(PUNCHER & kicker_toggle)
    {
        kicker_toggle = 0;
        BALL_DROP;
        ball_counter = 1; // Puncher (1 was too quick, 6 too slow)
        //ball_counter = 16; // Kicker was 17, 20, 18
        //ball_counter = (p3_wheel>>2)-12;
        punch_counter = 38;
    }
    if (!PUNCHER)
    {
        kicker_toggle = 1;
    }
    if (ball_counter == 1)
    {
        PUNCH;
    }
    if (ball_counter != 0)
    {
        ball_counter--;
    }
    //User_Mode_byte = (p3_wheel>>2)-12;
}



/*******************************************************************************
*	FUNCTION:		Arm_Position()
*	PURPOSE:		Position the Arm to pre-set angles
*                    Cap
*                    Knock-off
*                    Hurdle
*                    Carry
*                    Pickup
*                    Manual
*******************************************************************************/
void Arm_Position(void)
{
    static unsigned char counter=0;
    unsigned int desired_joint=0;

    ARM_MOTOR = 127; 
    if ((ARM_POSITION>>1) > 85 && (ARM_POSITION>>1) < 94)
    { // Manual
      ARM_MOTOR = ARM_DRIVE;
    }
    else if ((ARM_POSITION>>1) > 94 && (ARM_POSITION>>1) < 100)
    { // Pickup
        desired_joint = ARM_PICKUP;
        arm_movement(desired_joint, PICKUP);
        DropToRun = 0;
        //if (relay7_fwd==1) //If arms are open then pickup position
        //{
        //    desired_joint = ARM_PICKUP;
        //    arm_movement(desired_joint, PICKUP);
        //}
        //else  // arms closed go to pickup_1 position (dips lower)
        //{
        //    desired_joint = ARM_PICKUP_1;
        //    arm_movement(desired_joint, PICKUP);
        //}
    }
    else if ((ARM_POSITION>>1) > 100 && (ARM_POSITION>>1) < 106)
    { // Carry
        desired_joint = ARM_CARRY;
        arm_movement(desired_joint, CARRY);
        DropToRun = 0;
    }
    else if ((ARM_POSITION>>1) > 106 && (ARM_POSITION>>1) < 111)
    { // Knockoff
        desired_joint = ARM_KNOCKOFF;
        arm_movement(desired_joint, KNOCKOFF);
        DropToRun = 0;
    }
    else if ((ARM_POSITION>>1) > 111 && (ARM_POSITION>>1) < 118)
    { // Hurdle
        if (DropToRun == 0)
        {
            desired_joint = ARM_HURDLE;
            arm_movement(desired_joint, HURDLE);
        }
        else
        {
            desired_joint = ARM_CARRY;
            arm_movement(desired_joint, CARRY);
        }
    }
    else if ((ARM_POSITION>>1) > 118 && (ARM_POSITION>>1) < 127)
    { // Cap
        desired_joint = ARM_CAP;
        arm_movement(desired_joint, CAP);
        DropToRun = 0;
    }
    
    if ((ARM_POSITION>>1) >  94)
    { // Not Manual
	  // Safety override for limits of arm joints
	  if (ARM_POT < ARM_MIN && ARM_MOTOR > 127) 
	  {
		ARM_MOTOR = 127;
		printf("WARN ARM MIN LIMIT\r");
	  }
	  else if (ARM_POT > ARM_MAX && ARM_MOTOR < 127)
	  {
		ARM_MOTOR = 127;
		printf("WARN ARM MAX LIMIT\r");
	  }
    }

    // Okay, now we really mean it!
	// Safety override for limits of the pot itself
	if (ARM_POT < ARM_ABSOLUTE_MIN && ARM_MOTOR > 127) 
	{
		ARM_MOTOR = 127;
		printf("WARN POT ABSOLUTE MIN LIMIT\r");
	}
	else if (ARM_POT > ARM_ABSOLUTE_MAX && ARM_MOTOR < 127)
	{
		ARM_MOTOR = 127;
		printf("WARN POT ABSOLUTE MAX LIMIT\r");
	}

    counter++;
    if (counter == 38)
    {
      counter = 0;
      //printf("ARM_DRIVE=%d, ARM_POT=%d, desired=%d, ", (int)ARM_DRIVE, (int)ARM_POT, (int)desired_joint);
      //printf("ARM_LOWER_LIMIT=%d, ",(int)ARM_LOWER_LIMIT);
      //printf("offset=%d, ",(int)arm_offset);
      //printf("p4_aux>>1=%d\r\n", ARM_POSITION>>1);
    }
}


//*******************************************************************************
//* FUNCTION NAME: arm_movement
//* PURPOSE:       Position the arm at preset locations/angles
//*******************************************************************************
#define JOINT_DEADBAND 	2	// was 10
#define MAX_JOINT_SPEED 	127	// was 127
#define MIN_JOINT_SPEED 	0	// was 0 (KP overrides to hold position)
#define CAP_KP 11  // (13) worked w/o ball (6) bounced
#define HURDLE_KP 11  // 11 worked (15) too much w/o ball, 15 w/ball
#define CARRY_KP 9  // 11 worked w/o ball 
#define PICKUP_KP 15  // (15) worked w/o ball 
#define KNOCKOFF_KP 6  // (6) worked w/o ball 7 had trouble holding position

void arm_movement(unsigned int desired_joint, char position)
{
	unsigned int joint;
	long motor, kp;  

    if (position == CAP) kp = CAP_KP;
    if (position == HURDLE) kp = HURDLE_KP;
    if (position == CARRY) kp = CARRY_KP;
    if (position == PICKUP) kp = PICKUP_KP;
    if (position == KNOCKOFF) kp = KNOCKOFF_KP;

	//Current joint position
	joint = ARM_POT;

	if (joint < (desired_joint - JOINT_DEADBAND))
	{
		motor = (long)(desired_joint-joint)* kp/10;
		if(motor > MAX_JOINT_SPEED) 
		{
			motor = MAX_JOINT_SPEED;
		}
		else if (motor < MIN_JOINT_SPEED)
		{
			motor = MIN_JOINT_SPEED;
		}
		ARM_MOTOR = 127 - motor; // 0 is up & 254 is down
 	}
	else if(joint > (desired_joint + JOINT_DEADBAND))
	{
		motor = (long)(joint-desired_joint)* kp/10;
		if(motor > MAX_JOINT_SPEED) 
		{
			motor = MAX_JOINT_SPEED;
		}
		else if (motor < MIN_JOINT_SPEED)
		{
			motor = MIN_JOINT_SPEED;
		}
		ARM_MOTOR = 127 + motor;
	}
	else
	{
		ARM_MOTOR = 127;
	}
}
