/*******************************************************************************
*
*	TITLE:		teleop.h 
*
*	VERSION:	0.1 (Beta)                           
*
*	DATE:		31-Dec-2007
*
*	AUTHOR:		R. Kevin Watson
*				kevinw@jpl.nasa.gov
*
*	COMMENTS:	This file best viewed with tabs set to four.
*
*				You are free to use this source code for any non-commercial
*				use. Please do not make copies of this source code, modified
*				or un-modified, publicly available on the internet or elsewhere
*				without permission. Thanks.
*
*				Copyright 2007-2008 R. Kevin Watson. All rights are reserved.
*
********************************************************************************
*
*	Change log:
*
*	DATE         REV  DESCRIPTION
*	-----------  ---  ----------------------------------------------------------
*	31-Dec-2007  0.1  RKW Original
*
*******************************************************************************/
#ifndef _teleop_h
#define _teleop_h

#define LEFT_DRIVE p1_y
#define RIGHT_DRIVE p2_y
#define LEFT_MOTOR_1 pwm03
#define LEFT_MOTOR_2 pwm04
#define RIGHT_MOTOR_1 pwm01
#define RIGHT_MOTOR_2 pwm02
////////////////
#define XXLEFT_MOTOR_1 pwm15
#define XXLEFT_MOTOR_2 pwm16
#define XXRIGHT_MOTOR_1 pwm13
#define XXRIGHT_MOTOR_2 pwm14


#define IR_ON 	relay3_fwd=1; relay3_rev=0; Switch2_LED=1;
#define IR_OFF 	relay3_fwd=0; relay3_rev=1; Switch2_LED=0;

#define ARM_DRIVE p3_y
#define ARM_POSITION p4_aux
#define ARM_MOTOR pwm05
#define GRABBER		p3_sw_trig
#define PUNCHER		p3_sw_top
#define BALL_GRAB 	relay7_fwd=0; relay7_rev=0;
#define BALL_DROP 	relay7_fwd=1; relay7_rev=0;
#define BALL_TOGGLE relay7_fwd = !relay7_fwd; relay7_rev = !relay7_fwd;
//#define BALL_TOGGLE relay7_fwd = !relay7_fwd; relay7_rev = 0;
//#define BALL_GRAB 	relay7_fwd=1; relay7_rev=0;
//#define BALL_DROP 	relay7_fwd=0; relay7_rev=1;
#define PUNCH 	    relay5_fwd=1; relay5_rev=0;
#define RETRACT 	relay5_fwd=0; relay5_rev=0;
#define ARM_POT 	Get_ADC_Result(1)
#define ARM_LOWER_LIMIT rc_dig_in07

#define GEAR_SHIFT_LOW	p1_sw_top
#define GEAR_SHIFT_HIGH	p2_sw_top
#define SHIFT_HIGH	relay6_fwd = 1; relay6_rev  = 0;; // Shift into LOW gear
#define SHIFT_LOW	relay6_fwd = 0; relay6_rev  = 1;  // Shift into HIGH gear

#define COMPRESSOR relay8_fwd
#define PRESSURE		!rc_dig_in14
#define AUTOMODE ~PORTJ>>4
#define FIELD_LEFT   rc_dig_in11
#define FIELD_CENTER rc_dig_in12
#define FIELD_RIGHT  rc_dig_in13

#define START 1
#define IN_PROGRESS 2
#define COMPLETED 3
#define ARM_WAIT 4
#define DRIVE_SOME_MORE 5
#define GYROTURN 6

#define MANUAL  1
#define PICKUP  2
#define CARRY   3
#define HURDLE  4
#define KNOCKOFF 5
#define CAP     6

#define NOGYRO 0
#define GYRO   1


#define ARM_STOW        ((unsigned int)arm_offset)
#define ARM_PICKUP_1    ((unsigned int)120 + arm_offset)
#define ARM_PICKUP      ((unsigned int)200 + arm_offset)
#define ARM_CARRY       ((unsigned int)428 + arm_offset)
#define ARM_KNOCKOFF    ((unsigned int)920 + arm_offset)
#define ARM_HURDLE      ((unsigned int)1040 + arm_offset)
#define ARM_CAP         ((unsigned int)1040 + arm_offset)
// ARM_PICKUP puts the center of the rings 23" off the floor
// cap was 1060, knockoff was 899 then 860
// Cap = 998 ten 1020 (hurdle), knockoff=860 was 880
//ARMCAP was 1043
#define ARM_MIN         (arm_offset-100)
#define ARM_MAX         ((unsigned int)1100 + arm_offset)
#define ARM_ABSOLUTE_MIN 200
#define ARM_ABSOLUTE_MAX 3888

extern char GyroStatus;
extern unsigned int arm_offset;

// function prototypes
void Initialization(void);
void Teleop_Init(void);
void Teleop(void);
void Teleop_Spin(void);

void arm_movement(unsigned int, char);

#endif
